#version 330 core

uniform vec3 ambientColor;
uniform vec3 diffuseColor;
uniform vec3 specular;

uniform vec3 eye;

uniform vec3 lightDir;
uniform vec3 lightColor;

uniform mat4 depthMatrix;
uniform sampler2DShadow shadowMap;

uniform sampler2D diffuseTex;
uniform sampler2D specularTex;
uniform sampler2D normalTex;

out vec3 fragColor;

in vec3 tangentLightDir;
in vec3 tangentEyeDir;
in vec3 worldPosition;
in vec3 worldNormal;
in vec4 ShadowCoord;
in vec2 UV;

float sampleShadow( vec2 offSet, float bias ) {
	return texture( shadowMap, vec3(ShadowCoord.x + offSet.x / 4096.0, ShadowCoord.y + offSet.y / 4096.0, (ShadowCoord.z-bias)/ShadowCoord.w));
}

void main() {
	vec3 diffCol = diffuseColor * texture(diffuseTex, UV).rgb;
	vec3 specCol = specular * texture(specularTex, UV).rgb;
	
	float bias = 0.005;
	
	float shadow = 0;
	float x,y;
	for (y = -2.5 ; y <=2.5 ; y+=1.0)
		for (x = -2.5 ; x <=2.5 ; x+=1.0)
			shadow += sampleShadow(vec2(x,y),bias);
					
	shadow /= 25.0 ;

	vec3 texNormal = normalize(texture(normalTex, UV).rgb * 2 - 1);
	float diffuseTerm = clamp(dot(texNormal, -tangentLightDir), 0, 1);
	vec3 hVec = normalize(tangentEyeDir - tangentLightDir); 
	float specularTerm = pow(clamp(dot(texNormal, hVec), 0, 1), 50) * diffuseTerm;
	vec3 col = ambientColor * diffCol;
	col += diffCol * diffuseTerm * lightColor * (shadow + 0.4);
	col += specular * specularTerm * lightColor * (shadow + 0.4);
	
	fragColor = col;
}
